// klngram.cc
//
// Author: allauzen@cs.nyu.edu (Cyril Allauzen)
//
// \file
// Generate a n-gram kernel.

#include "kernel/lib/kernel.h"
#include "kernel/lib/rational-kernel.h"
#include "kernel/lib/composition-kernel.h"
#include "kernel/lib/ngram-fst.h"

using namespace kernel;
using namespace fst;

typedef VectorFst<LogArc> LogVectorFst;

DEFINE_int32(opt, -1, "rational function optimization level 0 to 5, -1 for default");
DEFINE_bool(normalize, true, "normalize rational kernel: k[x,y] / sqrt(k[x,x] * k[y,y])");
DEFINE_int32(order, -1, "n-gram order");
DEFINE_int32(sigma, -1, "alphabet size (i.e. max label)");
DEFINE_bool(gappy, false, "counts gappy n-gram");
DEFINE_int32(max_gap, -1, "max number of gaps allowed [default: order - 1]");
DEFINE_double(lambda, 0.1, "cost penalty for each gap");
DEFINE_int32(min_order, -1, "minimum order of n-gram to be counted [default: same as order]");
DEFINE_bool(cache_feature, true, "cache RKHS feature computation");
DEFINE_bool(cache_norm, false, "cache RKHS norm computation");
DEFINE_bool(cache_value, true, "cache kernel value computation");


int main(int argc, char** argv) {
  string usage = "Create an ngram kernel.\n\n  Usage: ";
  usage += argv[0];
  usage += " dataset > rational.kar \n";
  if (argc == 1) {
    cerr << argv[0] << ": use --help for usage" << endl;
    return 1;
  }

  std::set_new_handler(FailedNewHandler);
  SetFlags(usage.c_str(), &argc, &argv, true);
  if (argc < 2 || argc > 3) {
    ShowUsage();
    return 1;
  }

  if (FLAGS_order == -1)
    LOG(FATAL) << "klngram: n-gram order need to be specified "
               << "with --order=n\n";

  if (FLAGS_sigma == -1)
    LOG(FATAL) << "klngram: value of larger symbol need to be specified "
               << "with --sigma=n\n";

  LogVectorFst fst;
  if (FLAGS_gappy)
    GappyNgram(&fst, FLAGS_order, FLAGS_sigma, -log(FLAGS_lambda),
               FLAGS_max_gap == -1 ? FLAGS_order -1 : FLAGS_max_gap,
               FLAGS_min_order);
  else
    Ngram(&fst, FLAGS_order, FLAGS_sigma, FLAGS_min_order);

  ArcSort(&fst, ILabelCompare<LogArc>());
  RationalKernel<LogArc> kernel(
      fst,
      FLAGS_opt,
      FLAGS_normalize,
      (FLAGS_cache_feature ? kCacheFeature : 0) |
      (FLAGS_cache_norm ? kCacheNorm : 0) |
      (FLAGS_cache_value ? kCacheValue : 0));

  DataSet<LogFst> *data = new DataSet<LogFst>(string(argv[1]));

  kernel.SetData(data);
  kernel.Write(argc > 2 ? string(argv[2]) : string());
}
