// klsigmoid.cc
//
// Author: allauzen@cs.nyu.edu (Cyril Allauzen)
//
// \file
// Apply a sigmoid to a kernel.

#include "kernel/lib/kernel.h"
#include "kernel/lib/composition-kernel.h"
#include "kernel/lib/rational-kernel.h"

DEFINE_double(gamma, 1.0F, "gamma of sigmoid");
DEFINE_double(coef0, 0.0F, "coef0 of sigmoid");

using namespace kernel;

int main(int argc, char** argv) {
  string usage = "Compose kernel with sigmoid:";
  usage += " tanh(gamma * k[x,y] + coef0).\n\n  Usage: ";
  usage += argv[0];
  usage += " in.kar > out.kar\n";

  std::set_new_handler(FailedNewHandler);
  SetFlags(usage.c_str(), &argc, &argv, true);
  if (argc > 3) {
    ShowUsage();
    return 1;
  }
  string in = argc > 1 ? string(argv[1]) : string();
  string out = argc > 2 ? string(argv[2]) : string();

  // TODO: use KernelBase
  Kernel< Fst<LogArc> > *kernel = Kernel< Fst<LogArc> >::Read(in);
  if(!kernel)
    LOG(FATAL) << "klsigmoid: failed to read" << in;

  SigmoidKernel< Fst<LogArc> > k(kernel, FLAGS_gamma, FLAGS_coef0);
  k.Write(out);
  delete kernel;
  return 0;
}
